package trismegistoplanilla.mysqldao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.json.JSONArray;
import org.json.JSONObject;
import trismegistoplanilla.beans.DetalleLoteFichaDocenteBean;
import trismegistoplanilla.beans.LoteFichaBean;
import trismegistoplanilla.dao.DetalleLoteFichaDocenteDAO;
import trismegistoplanilla.utilities.CurrencyFormat;
import trismegistoplanilla.utilities.ResponseHelper;
import trismegistoplanilla.utilities.StringUtils;
import trismegistoplanilla.utilities.Variables;

public class DetalleLoteFichaDocenteMysqlDAO implements DetalleLoteFichaDocenteDAO {

	@Override
	public JSONObject listarDetalleLoteFichaDocenteDT(LoteFichaBean loteFicha) {
		System.out.println("DetalleLoteFichaDocenteMysqlDAO: listarDetalleLoteFichaDocenteDT");
		JSONObject jsonObjListar = null;
		JSONArray jsonArrayListar = new JSONArray();
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		ResponseHelper response = new ResponseHelper();
		try {
			String sql
				= "select "
				+ "personal.codper codigoPersona, "
				+ "ficha.codigo_ficha codigoFicha, "
				+ "personal.apellidoPaterno apellidoPaterno, "
				+ "personal.apellidoMaterno apellidoMaterno, "
				+ "personal.nombre nombre, "
				+ "personal.dni numeroDocumento, "
				+ "tipo_documento.descripcion_corta tipodocumento, "
				+ "date_format(ficha_laboral.fecha_ingreso,'%d/%m/%Y') fechaInicio, "
				+ "upper(monthname(ficha_laboral.fecha_ingreso)) mes, "
				+ "sueldo_docente.codigo_area_cargo_tipo_pago as tipoPago, "
				+ "ifnull(sueldo_docente.costo_a,'') costoADocente, "
				+ "ifnull(sueldo_docente.costo_b,'') costoBDocente, "
				+ "ifnull(sueldo_docente.costo_c,'') costoCDocente, "
				+ "ifnull(sueldo_docente.costo_mensual,'') costoMensualDocente,"
				+ "ficha_laboral.tipo_ficha "
				+ "FROM ficha_laboral "
				+ "inner join ficha ON ficha.codigo_ficha = ficha_laboral.codigo_ficha "
				+ "inner join estado_ficha on estado_ficha.codigo_ficha = ficha.codigo_ficha "
				+ "inner join personal ON personal.codper = ficha.codigo_persona "
				+ "inner join tipo_documento ON tipo_documento.codigo_tipo_documento = personal.codigo_tipo_documento "
				+ "left join sueldo_docente ON sueldo_docente.codigo_ficha = ficha_laboral.codigo_ficha "
				+ "where estado_ficha.estado_registro = 1 "
				+ "and estado_ficha.codigo_tipo_estado_ficha = 4 "
				+ "and sueldo_docente.estado_registro = 1 ";
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			ps = cnx.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				DetalleLoteFichaDocenteBean detalleLoteFichaDocente = new DetalleLoteFichaDocenteBean();
				detalleLoteFichaDocente.setCodigoFicha(rs.getInt("codigoFicha"));
				detalleLoteFichaDocente.setApellidoPaterno(rs.getString("apellidoPaterno"));
				detalleLoteFichaDocente.setApellidoMaterno(rs.getString("apellidoMaterno"));
				detalleLoteFichaDocente.setNombre(rs.getString("nombre"));
				detalleLoteFichaDocente.setNumeroDocumento(rs.getString("numeroDocumento"));
				detalleLoteFichaDocente.setTipoDocumento(rs.getString("tipodocumento"));
				detalleLoteFichaDocente.setFechaInicio(rs.getString("fechaInicio"));
				detalleLoteFichaDocente.setCodigoPago(rs.getInt("tipoPago"));
				detalleLoteFichaDocente.setMes(rs.getString("mes"));
				detalleLoteFichaDocente.setCostoa(CurrencyFormat.getCustomCurrency(rs.getDouble("costoADocente")));
				detalleLoteFichaDocente.setCostob(CurrencyFormat.getCustomCurrency(rs.getDouble("costoBDocente")));
				detalleLoteFichaDocente.setCostoc(CurrencyFormat.getCustomCurrency(rs.getDouble("costoCDocente")));
				detalleLoteFichaDocente.setCostoMensual(CurrencyFormat.getCustomCurrency(rs.getDouble("costoMensualDocente")));
				JSONObject jsonObjDetalleLoteFichaDocente = new JSONObject(detalleLoteFichaDocente);
				jsonObjDetalleLoteFichaDocente.put("codigoPersona", rs.getInt("codigoPersona"));
				jsonObjDetalleLoteFichaDocente.put("tipoFicha", rs.getString("tipo_ficha"));
				jsonArrayListar.put(jsonObjDetalleLoteFichaDocente);
			}

			JSONObject jsonObjDetalleLoteFichaDocente = new JSONObject();
			jsonObjDetalleLoteFichaDocente.put("fichas", jsonArrayListar);

			response.setStatus(true);
			response.setMessage("Se ha listado las fichas de docentes correctamente!");
			response.setData(jsonObjDetalleLoteFichaDocente);

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		jsonObjListar = new JSONObject(response);
		return jsonObjListar;
	}

	@Override
	public JSONObject registrarSueldosPresidenciaLoteDocente(JSONObject data) {
		System.out.println("DetalleLoteFichaDocenteMysqlDAO: registrarSueldosPresidenciaLoteDocente");
		JSONObject jsonObjRegistrarSueldoPresidencia = null;
		Connection cnx = null;
		PreparedStatement psDesactivarEstadoSueldoDocente = null,
			psInsertarSueldoDocente = null,
			psDesactivarEstadoLote = null,
			psInsertarEstadoLote = null,
			psDesactivarEstadoFicha = null,
			psInsertarEstadoFicha = null;
		ResponseHelper response = new ResponseHelper();
		String sql = "";
		JSONArray jsonArrayFichas = data.getJSONArray("fichas");
		int longitudJsonArrayFichas = jsonArrayFichas.length();

		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			cnx.setAutoCommit(false);
			sql
				= "update sueldo_docente "
				+ "set estado_registro = 0 "
				+ "where sueldo_docente.estado_registro = 1 "
				+ "and codigo_ficha = ?";
			psDesactivarEstadoSueldoDocente = cnx.prepareStatement(sql);

			for (int i = 0; i < longitudJsonArrayFichas; i++) {
				psDesactivarEstadoSueldoDocente.setInt(1, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
				psDesactivarEstadoSueldoDocente.addBatch();
			}
			int resultadoDesactivarEstadoSueldoDocente[] = psDesactivarEstadoSueldoDocente.executeBatch();
			int totalFilasDesactivarEstadoSueldoDocente = resultadoDesactivarEstadoSueldoDocente.length;

			if (totalFilasDesactivarEstadoSueldoDocente != 0) {
				sql
					= "insert into sueldo_docente "
					+ "(codigo_ficha, codigo_area_cargo_tipo_pago, "
					+ "costo_mensual, costo_a, costo_b, costo_c, "
					+ "observacion, fecha_registro, estado_registro) "
					+ "values (?, ?, ?, ?, ?, ?, LTRIM(RTRIM(UPPER(?))), now(), 1)";
				psInsertarSueldoDocente = cnx.prepareStatement(sql);

				for (int i = 0; i < longitudJsonArrayFichas; i++) {
					int tipoPago = jsonArrayFichas.getJSONObject(i).getInt("codigoPago");
					psInsertarSueldoDocente.setInt(1, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
					psInsertarSueldoDocente.setInt(2, tipoPago);
					if (tipoPago == 5) { // Si el pago es horas, costo mensual es null
						psInsertarSueldoDocente.setObject(3, null);
					} else {
						psInsertarSueldoDocente.setString(3, jsonArrayFichas.getJSONObject(i).getString("costoMensual"));
					}
					if (tipoPago == 6) { // Si el pago es mensual, costo por secciones es null
						psInsertarSueldoDocente.setObject(4, null);
						psInsertarSueldoDocente.setObject(5, null);
						psInsertarSueldoDocente.setObject(6, null);
					} else {
						psInsertarSueldoDocente.setString(4, jsonArrayFichas.getJSONObject(i).getString("costoa"));
						psInsertarSueldoDocente.setString(5, jsonArrayFichas.getJSONObject(i).getString("costob"));
						psInsertarSueldoDocente.setString(6, jsonArrayFichas.getJSONObject(i).getString("costoc"));
					}
					psInsertarSueldoDocente.setString(7, jsonArrayFichas.getJSONObject(i).getString("observacion"));
					psInsertarSueldoDocente.addBatch();
				}

				int resultadoInsertarSueldoDocente[] = psInsertarSueldoDocente.executeBatch();
				int totalFilasInsetarSueldoDocente = resultadoInsertarSueldoDocente.length;

				if (totalFilasInsetarSueldoDocente != 0) {

					sql
						= "update estado_ficha_lote "
						+ "set estado_registro = 0 "
						+ "where codigo_ficha_lote = ? "
						+ "and estado_ficha_lote.estado_registro = 1";
					psDesactivarEstadoLote = cnx.prepareStatement(sql);
					psDesactivarEstadoLote.setInt(1, data.getInt("codigoLote"));
					int resultadoDesactivarEstadoLote = psDesactivarEstadoLote.executeUpdate();
					if (resultadoDesactivarEstadoLote != 0) {
						sql
							= "insert into estado_ficha_lote "
							+ "(codigo_ficha_lote, "
							+ "codigo_tipo_estado_ficha_lote, "
							+ "fecha_registro, "
							+ "codigo_usuario, "
							+ "estado_registro) "
							+ "VALUES (?,?,now(),?,1)";
						psInsertarEstadoLote = cnx.prepareStatement(sql);
						psInsertarEstadoLote.setInt(1, data.getInt("codigoLote"));
						if (!data.getBoolean("estadoLote")) { // No cambio los sueldos
							psInsertarEstadoLote.setInt(2, 2); // Estado completado!
						} else { // Si cambio los sueldos
							psInsertarEstadoLote.setInt(2, 3); // Completado con obs
						}
						psInsertarEstadoLote.setInt(3, data.getInt("usuario"));
						int resultadoInsertarEstadoLote = psInsertarEstadoLote.executeUpdate();

						if (resultadoInsertarEstadoLote != 0) {
							sql
								= "update estado_ficha "
								+ "set estado_registro = 0 "
								+ "where estado_ficha.estado_registro = 1 "
								+ "and estado_ficha.codigo_ficha = ?";
							psDesactivarEstadoFicha = cnx.prepareStatement(sql);

							for (int i = 0; i < longitudJsonArrayFichas; i++) {
								psDesactivarEstadoFicha.setInt(1, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
								psDesactivarEstadoFicha.addBatch();
							}
							int resultadoDesactivarEstadoFicha[] = psDesactivarEstadoFicha.executeBatch();
							int totalFilasDesactivarEstadoFicha = resultadoDesactivarEstadoFicha.length;

							if (totalFilasDesactivarEstadoFicha != 0) {
								sql
									= "insert into estado_ficha "
									+ "(codigo_ficha, "
									+ "codigo_tipo_estado_ficha, "
									+ "fecha_registro, "
									+ "codigo_usuario, "
									+ "estado_registro) "
									+ "VALUES (?, ? ,now() ,?  ,1)";
								psInsertarEstadoFicha = cnx.prepareStatement(sql);

								for (int i = 0; i < longitudJsonArrayFichas; i++) {
									psInsertarEstadoFicha.setInt(1, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
									if (!jsonArrayFichas.getJSONObject(i).getBoolean("estadoFicha")) { // NO cambio el sueldo de la ficha
										psInsertarEstadoFicha.setInt(2, 7); // Estado: Sueldo aprobado por presidencia
									} else { // Si cambio el sueldo de la ficha
										psInsertarEstadoFicha.setInt(2, 8); // Estado: Observado por presidencia: Por evaluar.
									}

									psInsertarEstadoFicha.setInt(3, data.getInt("usuario"));
									psInsertarEstadoFicha.addBatch();
								}
								int resultadoInsertarEstadoFicha[] = psInsertarEstadoFicha.executeBatch();
								int totalFilasInsertarEstadoFicha = resultadoInsertarEstadoFicha.length;

								if (totalFilasInsertarEstadoFicha != 0) {
									response.setStatus(true);
									response.setMessage("Se actualizó correctamente el lote!");
									cnx.commit();
								} else {
									response.setStatus(false);
									response.setMessage("Lo sentimos, no se pudo insertar el estado de la ficha");
									cnx.rollback();
								}
							} else {
								response.setStatus(false);
								response.setMessage("Lo sentimos, no se pudo desactivar los estados de la ficha");
								cnx.rollback();
							}

						} else {
							response.setStatus(false);
							response.setMessage("Lo sentimos, no se pudo insertar el estado del lote");
							cnx.rollback();
						}

					} else {
						response.setStatus(false);
						response.setMessage("Lo sentimos, no se pudo desactivar el estado del lote");
						cnx.rollback();
					}

				} else {
					response.setStatus(false);
					response.setMessage("Lo sentimos, no se pudo insertar los estados de los sueldos");
					cnx.rollback();
				}

			} else {
				response.setStatus(false);
				response.setMessage("Lo sentimos, no se pudo desactivar los estados de los sueldos");
				cnx.rollback();
			}

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (psInsertarEstadoFicha != null) {
					psInsertarEstadoFicha.close();
				}
				if (psDesactivarEstadoFicha != null) {
					psDesactivarEstadoFicha.close();
				}
				if (psInsertarEstadoLote != null) {
					psInsertarEstadoLote.close();
				}
				if (psDesactivarEstadoLote != null) {
					psDesactivarEstadoLote.close();
				}
				if (psInsertarSueldoDocente != null) {
					psInsertarSueldoDocente.close();
				}
				if (psDesactivarEstadoSueldoDocente != null) {
					psDesactivarEstadoSueldoDocente.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jsonObjRegistrarSueldoPresidencia = new JSONObject(response);
		return jsonObjRegistrarSueldoPresidencia;

	}

	@Override
	public JSONObject listarDetalleLoteFichaAdministrativoDT(JSONObject data) {
		System.out.println("DetalleLoteFichaDocenteMysqlDAO: listarDetalleLoteFichaAdministrativoDT");
		JSONObject jsonReturn = null;
		JSONArray jsonArray = new JSONArray();
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		ResponseHelper response = new ResponseHelper();
		String sql;
		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			sql = ""
				+ "select "
				+ "personal.codper codigoPersona, "
				+ "ficha.codigo_ficha codigoFicha, "
				+ "personal.apellidoPaterno apellidoPaterno, "
				+ "personal.apellidoMaterno apellidoMaterno, "
				+ "personal.nombre nombre, "
				+ "personal.dni numeroDocumento, "
				+ "sueldo_administrativo.sueldo_escalafon escalafon, "
				+ "sueldo_administrativo.sueldo_mensual costoMensual, "
				+ "tipo_documento.descripcion_corta tipodocumento, "
				+ "date_format(ficha_laboral.fecha_ingreso,'%d/%m/%Y') fechaInicio, "
				+ "upper(monthname(ficha_laboral.fecha_ingreso)) mes, "
				+ "sueldo_administrativo.codigo_area_cargo_tipo_pago codigoAreaCargoTipoPago, "
				+ "ficha_laboral.tipo_ficha "
				+ "FROM ficha_laboral "
				+ "inner join ficha ON ficha.codigo_ficha = ficha_laboral.codigo_ficha "
				+ "inner join estado_ficha on estado_ficha.codigo_ficha = ficha.codigo_ficha "
				+ "inner join personal ON personal.codper = ficha.codigo_persona "
				+ "inner join tipo_documento ON tipo_documento.codigo_tipo_documento = personal.codigo_tipo_documento "
				+ "left join sueldo_administrativo ON sueldo_administrativo.codigo_ficha = ficha_laboral.codigo_ficha "
				+ "where estado_ficha.estado_registro = 1 "
				+ "and estado_ficha.codigo_tipo_estado_ficha = 4 "
				+ "and sueldo_administrativo.estado_registro = 1 ";
			ps = cnx.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				DetalleLoteFichaDocenteBean detalleLote = new DetalleLoteFichaDocenteBean();
				detalleLote.setCodigoFicha(rs.getInt("codigoFicha"));
				detalleLote.setApellidoPaterno(rs.getString("apellidoPaterno"));
				detalleLote.setApellidoMaterno(rs.getString("apellidoMaterno"));
				detalleLote.setNombre(rs.getString("nombre"));
				detalleLote.setNumeroDocumento(rs.getString("numeroDocumento"));
				detalleLote.setTipoDocumento(rs.getString("tipodocumento"));
				detalleLote.setFechaInicio(rs.getString("fechaInicio"));
				detalleLote.setMes(rs.getString("mes"));
				JSONObject jsonObj = new JSONObject(detalleLote);
				jsonObj.put("sueldoEscalafon", CurrencyFormat.getCustomCurrency(rs.getDouble("escalafon")));
				jsonObj.put("sueldoMensual", CurrencyFormat.getCustomCurrency(rs.getDouble("costoMensual")));
				jsonObj.put("codigoAreaCargoTipoPago", rs.getInt("codigoAreaCargoTipoPago"));
				jsonObj.put("codigoPersona", rs.getInt("codigoPersona"));
				jsonObj.put("tipoFicha", rs.getString("tipo_ficha"));
				jsonArray.put(jsonObj);
			}

			JSONObject jsonObj = new JSONObject();
			jsonObj.put("fichas", jsonArray);

			response.setStatus(true);
			response.setMessage("Se ha listado las fichas administrativas correctamente");
			response.setData(jsonObj);
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jsonReturn = new JSONObject(response);
		return jsonReturn;
	}

	@Override
	public JSONObject registrarSueldosPresidenciaLoteAdministrativo(JSONObject data) {
		System.out.println("DetalleLoteFichaDocenteMysqlDAO: registrarSueldosPresidenciaLoteAdministrativo");
		JSONObject jsonObjRegistrarSueldoPresidencia = null;
		Connection cnx = null;
		PreparedStatement ps = null;
		ResponseHelper response = new ResponseHelper();
		String sql = "";
		JSONArray jsonArrayFichas = data.getJSONArray("fichas");
		int longitudJsonArrayFichas = jsonArrayFichas.length();
		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			cnx.setAutoCommit(false);
			sql
				= "update sueldo_administrativo "
				+ "set estado_registro = 0 "
				+ "where estado_registro = 1 "
				+ "and codigo_ficha = ?";
			ps = cnx.prepareStatement(sql);

			for (int i = 0; i < longitudJsonArrayFichas; i++) {
				ps.setInt(1, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
				ps.addBatch();
			}
			int resultadoDesactivarEstadoSueldoAdministrativo[] = ps.executeBatch();
			int totalFilasDesactivarEstadoSueldoAdministrativo = resultadoDesactivarEstadoSueldoAdministrativo.length;

			if (totalFilasDesactivarEstadoSueldoAdministrativo != 0) {
				sql
					= "insert into sueldo_administrativo "
					+ "(codigo_ficha, codigo_area_cargo_tipo_pago, "
					+ "sueldo_escalafon, sueldo_mensual, sueldo_presidencia, observacion, "
					+ "fecha_registro, estado_registro) "
					+ "values (?,?,?,?,?,UPPER(RTRIM(LTRIM(?))), now(),1)";
				ps = cnx.prepareStatement(sql);

				for (int i = 0; i < longitudJsonArrayFichas; i++) {
					int c = 1;
					ps.setInt(c++, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
					ps.setInt(c++, jsonArrayFichas.getJSONObject(i).getInt("codigoAreaCargoTipoPago"));
					ps.setString(c++, jsonArrayFichas.getJSONObject(i).getString("sueldoEscalafon"));
					ps.setString(c++, jsonArrayFichas.getJSONObject(i).getString("sueldoMensual"));
					ps.setString(c++, jsonArrayFichas.getJSONObject(i).getString("sueldoPresidencia"));
					if (jsonArrayFichas.getJSONObject(i).getBoolean("estadoFicha")) {
						ps.setString(c++, jsonArrayFichas.getJSONObject(i).getString("observacion"));
					} else {
						ps.setObject(c++, null);
					}
					ps.addBatch();
				}

				int resultadoInsertarSueldoAdministrativo[] = ps.executeBatch();
				int totalFilasInsetarSueldoAdministrativo = resultadoInsertarSueldoAdministrativo.length;

				if (totalFilasInsetarSueldoAdministrativo != 0) {

					sql
						= "update estado_ficha_lote "
						+ "set estado_registro = 0 "
						+ "where codigo_ficha_lote = ? "
						+ "and estado_ficha_lote.estado_registro = 1";
					ps = cnx.prepareStatement(sql);
					ps.setInt(1, data.getInt("codigoLote"));
					int resultadoDesactivarEstadoLote = ps.executeUpdate();

					if (resultadoDesactivarEstadoLote != 0) {
						sql
							= "insert into estado_ficha_lote "
							+ "(codigo_ficha_lote, "
							+ "codigo_tipo_estado_ficha_lote, "
							+ "fecha_registro, "
							+ "codigo_usuario, "
							+ "estado_registro) "
							+ "VALUES (?,?,now(),?,1)";
						ps = cnx.prepareStatement(sql);

						ps.setInt(1, data.getInt("codigoLote"));
						if (!data.getBoolean("estadoLote")) { // No cambio los sueldos
							ps.setInt(2, 2); // Estado completado!
						} else { // Si cambio los sueldos
							ps.setInt(2, 3); // Completado con obs
						}
						ps.setInt(3, data.getInt("usuario"));
						int resultadoInsertarEstadoLote = ps.executeUpdate();

						if (resultadoInsertarEstadoLote != 0) {
							sql
								= "update estado_ficha "
								+ "set estado_registro = 0 "
								+ "where estado_ficha.estado_registro = 1 "
								+ "and estado_ficha.codigo_ficha = ?";
							ps = cnx.prepareStatement(sql);

							for (int i = 0; i < longitudJsonArrayFichas; i++) {
								ps.setInt(1, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
								ps.addBatch();
							}
							int resultadoDesactivarEstadoFicha[] = ps.executeBatch();
							int totalFilasDesactivarEstadoFicha = resultadoDesactivarEstadoFicha.length;

							if (totalFilasDesactivarEstadoFicha != 0) {
								sql
									= "insert into estado_ficha "
									+ "(codigo_ficha, "
									+ "codigo_tipo_estado_ficha, "
									+ "fecha_registro, "
									+ "codigo_usuario, "
									+ "estado_registro) "
									+ "VALUES (?, ? ,now() ,?  ,1)";
								ps = cnx.prepareStatement(sql);

								for (int i = 0; i < longitudJsonArrayFichas; i++) {
									ps.setInt(1, jsonArrayFichas.getJSONObject(i).getInt("codigoFicha"));
									if (!jsonArrayFichas.getJSONObject(i).getBoolean("estadoFicha")) { // NO cambio el sueldo de la ficha
										ps.setInt(2, 7); // Estado: Sueldo aprobado por presidencia
									} else { // Si cambio el sueldo de la ficha
										ps.setInt(2, 8); // Estado: Observado por presidencia: Por evaluar.
									}

									ps.setInt(3, data.getInt("usuario"));
									ps.addBatch();
								}
								int resultadoInsertarEstadoFicha[] = ps.executeBatch();
								int totalFilasInsertarEstadoFicha = resultadoInsertarEstadoFicha.length;

								if (totalFilasInsertarEstadoFicha != 0) {
									response.setStatus(true);
									response.setMessage("Se actualizó correctamente el lote!");
									cnx.commit();
								} else {
									response.setStatus(false);
									response.setMessage("Lo sentimos, no se pudo insertar el estado de la ficha");
									cnx.rollback();
								}
							} else {
								response.setStatus(false);
								response.setMessage("Lo sentimos, no se pudo desactivar los estados de la ficha");
								cnx.rollback();
							}

						} else {
							response.setStatus(false);
							response.setMessage("Lo sentimos, no se pudo insertar el estado del lote");
							cnx.rollback();
						}

					} else {
						response.setStatus(false);
						response.setMessage("Lo sentimos, no se pudo desactivar el estado del lote");
						cnx.rollback();
					}

				} else {
					response.setStatus(false);
					response.setMessage("Lo sentimos, no se pudo insertar los estados de los sueldos");
					cnx.rollback();
				}

			} else {
				response.setStatus(false);
				response.setMessage("Lo sentimos, no se pudo desactivar los estados de los sueldos");
				cnx.rollback();
			}

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jsonObjRegistrarSueldoPresidencia = new JSONObject(response);
		return jsonObjRegistrarSueldoPresidencia;
	}

	@Override
	public JSONObject obtenerDetalleFicha(JSONObject data) {
		System.out.println("DetalleLoteFichaDocenteMysqlDAO: obtenerDetalleFicha");
		JSONObject jsonReturn = null;
		ResponseHelper response = new ResponseHelper();
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		String sql, columns, inners;
		if (data.getString("tipoFicha").equals("A")) {
			columns = "sueldo_administrativo.sueldo_escalafon sueldoEscalafon, "
				+ "sueldo_administrativo.sueldo_mensual sueldoMensual, "
				+ "sueldo_administrativo.observacion observacion, ";
			inners = "inner join sueldo_administrativo ON sueldo_administrativo.codigo_ficha = ficha_laboral.codigo_ficha "
				+ "inner join area_cargo_tipo_pago ON area_cargo_tipo_pago.codigo_area_cargo_tipo_pago = sueldo_administrativo.codigo_area_cargo_tipo_pago ";
		} else {
			columns = "sueldo_docente.costo_mensual costoMensual, "
				+ "sueldo_docente.costo_a costoa, "
				+ "sueldo_docente.costo_b costob, "
				+ "sueldo_docente.costo_c costoc, "
				+ "sueldo_docente.observacion observacion, ";
			inners = "inner join sueldo_docente ON sueldo_docente.codigo_ficha = ficha_laboral.codigo_ficha "
				+ "inner join area_cargo_tipo_pago ON area_cargo_tipo_pago.codigo_area_cargo_tipo_pago = sueldo_docente.codigo_area_cargo_tipo_pago ";
		}
		try {
			sql
				= "select "
				+ "ficha_laboral.codigo_ficha codigoFicha, "
				+ "date_format(ficha_laboral.fecha_ingreso,'%d/%m/%Y') fechaIngreso, "
				+ "date_format(ficha_laboral.fecha_fin,'%d/%m/%Y') fechaTermino, "
				+ columns
				+ "tipo_pago.nombre tipoPago, "
				+ "sede.deslocal sede, "
				+ "area.nombre area, "
				+ "cargo.nombre cargo "
				+ "FROM ficha_laboral "
				+ "inner join sede_area ON sede_area.codigo_sede_area = ficha_laboral.codigo_sede_area "
				+ "inner join locales sede ON sede.codigo_sede = sede_area.codigo_sede "
				+ "inner join area_cargo ON area_cargo.codigo_area_cargo = ficha_laboral.codigo_area_cargo "
				+ "inner join area ON area.codigo_area = area_cargo.codigo_area "
				+ "inner join cargo ON cargo.codigo_cargo = area_cargo.codigo_cargo "
				+ inners
				+ "inner join tipo_pago ON tipo_pago.codigo_tipo_pago = area_cargo_tipo_pago.codigo_tipo_pago "
				+ "where ficha_laboral.codigo_ficha = ?";
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			ps = cnx.prepareStatement(sql);
			ps.setInt(1, data.getInt("codigoFicha"));
			rs = ps.executeQuery();
			if (rs.next()) {
				JSONObject jsonObj = new JSONObject();
				jsonObj.put("codigoFicha", rs.getInt("codigoFicha"));
				jsonObj.put("fechaIngreso", rs.getString("fechaIngreso"));
				jsonObj.put("fechaTermino", rs.getString("fechaTermino"));
				if (data.getString("tipoFicha").equals("A")) {
					jsonObj.put("sueldoEscalafon", StringUtils.isNullOrEmpty(rs.getString("sueldoEscalafon")) ? "0.00" : CurrencyFormat.getCustomCurrency(rs.getDouble("sueldoEscalafon")));
					jsonObj.put("sueldoMensual", StringUtils.isNullOrEmpty(rs.getString("sueldoMensual")) ? "0.00" : CurrencyFormat.getCustomCurrency(rs.getDouble("sueldoMensual")));
				} else {
					jsonObj.put("costoMensual", StringUtils.isNullOrEmpty(rs.getString("costoMensual")) ? "0.00" : CurrencyFormat.getCustomCurrency(rs.getDouble("costoMensual")));
					jsonObj.put("costoa", StringUtils.isNullOrEmpty(rs.getString("costoa")) ? "0.00" : CurrencyFormat.getCustomCurrency(rs.getDouble("costoa")));
					jsonObj.put("costob", StringUtils.isNullOrEmpty(rs.getString("costob")) ? "0.00" : CurrencyFormat.getCustomCurrency(rs.getDouble("costob")));
					jsonObj.put("costoc", StringUtils.isNullOrEmpty(rs.getString("costoc")) ? "0.00" : CurrencyFormat.getCustomCurrency(rs.getDouble("costoc")));
				}
				jsonObj.put("observacion", StringUtils.isNullOrEmpty(rs.getString("observacion")) ? "" : rs.getString("observacion"));
				jsonObj.put("tipoPago", rs.getString("tipoPago"));
				jsonObj.put("sede", rs.getString("sede"));
				jsonObj.put("area", rs.getString("area"));
				jsonObj.put("cargo", rs.getString("cargo"));

				response.setStatus(true);
				response.setMessage("Se obtuvo los datos administrativos correctamente");
				response.setData(jsonObj);
			} else {
				response.setStatus(false);
				response.setMessage("No se encontraron datos administrativos.");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jsonReturn = new JSONObject(response);
		return jsonReturn;
	}
}
