package trismegistoplanilla.mysqldao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.json.JSONArray;
import org.json.JSONObject;
import trismegistoplanilla.beans.NacionalidadBean;
import trismegistoplanilla.dao.NacionalidadDAO;
import trismegistoplanilla.utilities.ResponseHelper;
import trismegistoplanilla.utilities.Variables;

public class NacionalidadMysqlDAO implements NacionalidadDAO {

	@Override
	public JSONObject listarNacionalidad() {
		System.out.println("NacionalidadMysqlDAO: listarNacionalidad");
		JSONObject jsonListarNacionalidad = null;
		JSONArray jsonArrayListarNacionalidad = new JSONArray();
		Connection connection = null;
		PreparedStatement ps = null;
		ResultSet rs = null;

		ResponseHelper response = new ResponseHelper();
		try {
			connection = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			String sql
				= "select codigo_nacionalidad codigoNacionalidad, "
				+ "pais pais, "
				+ "gentilicio gentilicio, "
				+ "iso iso, "
				+ "estado_registro estadoRegistro "
				+ "FROM nacionalidad where estado_registro = 1";
			ps = connection.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				NacionalidadBean nacionalidad = new NacionalidadBean();
				nacionalidad.setCodigoNacionalidad(rs.getInt("codigoNacionalidad"));
				nacionalidad.setPais(rs.getString("pais"));
				nacionalidad.setGentilicio(rs.getString("gentilicio"));
				nacionalidad.setIso(rs.getString("iso"));
				nacionalidad.setEstadoRegistro(rs.getInt("estadoRegistro"));
				JSONObject jsonObjListarNacionalidad = new JSONObject(nacionalidad);
				jsonArrayListarNacionalidad.put(jsonObjListarNacionalidad);
			}

			JSONObject jsonObjListarNacionalidad = new JSONObject();
			jsonObjListarNacionalidad.put("Nacionalidades", jsonArrayListarNacionalidad);
			response.setStatus(true);
			response.setMessage("Las nacionalidades se han listado correctamente");
			response.setData(jsonObjListarNacionalidad);

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (connection != null) {
					connection.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		jsonListarNacionalidad = new JSONObject(response);
		return jsonListarNacionalidad;
	}

	@Override
	public JSONObject validarExistenciaNacionalidad(NacionalidadBean nacionalidad) {
		System.out.println("NacionalidadMysqlDAO: validarExistenciaNacionalidad");
		JSONObject jsonvalidadExistenciaNacionalidad = null;
		ResponseHelper response = new ResponseHelper();
		int existeEstadoCivil = 0;

		Connection connection = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			connection = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			String sql
				= "select "
				+ "count(1) as existeNacionalidad "
				+ "from nacionalidad "
				+ "where codigo_nacionalidad = ? and estado_registro = 1";
			ps = connection.prepareStatement(sql);
			ps.setInt(1, nacionalidad.getCodigoNacionalidad());
			rs = ps.executeQuery();
			rs.next();
			existeEstadoCivil = rs.getInt("existeNacionalidad");
			if (existeEstadoCivil > 0) {
				response.setStatus(true);
				response.setMessage("La nacionalidad seleccionada existe.");
			} else {
				response.setStatus(false);
				response.setMessage("La nacionalidad seleccionada no existe.");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (connection != null) {
					connection.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		jsonvalidadExistenciaNacionalidad = new JSONObject(response);
		return jsonvalidadExistenciaNacionalidad;
	}

}
