package trismegistoplanilla.mysqldao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.json.JSONObject;
import trismegistoplanilla.beans.TokenFichaBean;
import trismegistoplanilla.dao.TokenFichaDAO;
import trismegistoplanilla.utilities.ResponseHelper;
import trismegistoplanilla.utilities.Variables;

public class TokenFichaMysqlDAO implements TokenFichaDAO {

	@Override
	public JSONObject validarTokenURL(TokenFichaBean tf) {
		System.out.println("TokenFichaMysqlDAO: validarTokenURL");
		JSONObject jsonObjValidarTokenURL = null;
		ResponseHelper response = new ResponseHelper();
		int tokenValidoURL = 0;

		String sql = ""
			+ "select "
			+ "count(1) tokenValidoURL "
			+ "from token_ficha "
			+ "where codigo_token_ficha = ? and "
			+ "token = ? ";
		
		Connection conexion = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			conexion = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			ps = conexion.prepareStatement(sql);
			ps.setInt(1, tf.getCodigoTokenFicha());
			ps.setString(2, tf.getToken());
			rs = ps.executeQuery();
			rs.next();
			tokenValidoURL = rs.getInt("tokenValidoURL");
			if (tokenValidoURL > 0) {
				response.setStatus(true);
				response.setMessage("El token es válido");
			} else {
				response.setStatus(false);
				// error token no valido
				response.setMessage("El token no existe o ha caducado");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (conexion != null) {
					conexion.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		jsonObjValidarTokenURL = new JSONObject(response);
		return jsonObjValidarTokenURL;
	}

	@Override
	public JSONObject validarToken(TokenFichaBean tf) {
		System.out.println("TokenFichaMysqlDAO: validarToken");
		JSONObject jsonObjValidarToken = null;
		ResponseHelper response = new ResponseHelper();
		int tokenValido = 0;

		String sql = ""
			+ "select "
			+ "count(1) tokenValido "
			+ "from token_ficha "
			+ "where "
			+ "codigo_token_ficha = ? and "
			+ "token = ? and "
			+ "fecha_expiracion >= now() and  "
			+ "estado_registro = 1 ";
		
		Connection conexion = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			conexion = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			ps = conexion.prepareStatement(sql);
			ps.setInt(1, tf.getCodigoTokenFicha());
			ps.setString(2, tf.getToken());
			rs = ps.executeQuery();
			rs.next();
			tokenValido = rs.getInt("tokenValido");
			if (tokenValido > 0) {
				response.setStatus(true);
				response.setMessage("El token es válido");
			} else {
				response.setStatus(false);
				response.setMessage("Error, el token no existe o esta inhabilitado");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (conexion != null) {
					conexion.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		jsonObjValidarToken = new JSONObject(response);
		return jsonObjValidarToken;
	}

	@Override
	public JSONObject desactivarToken(TokenFichaBean tf) {
		System.out.println("TokenFichaMysqlDAO: desactivarToken");
		JSONObject jsonObjDesactivarToken = null;
		ResponseHelper response = new ResponseHelper();

		String sql = ""
			+ "update token_ficha "
			+ "set estado_registro = 0 "
			+ "where codigo_token_ficha = ? and estado_registro = 1";
		
		Connection conexion = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			conexion = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			conexion.setAutoCommit(false);
			ps = conexion.prepareStatement(sql);
			ps.setInt(1, tf.getCodigoTokenFicha());
			int desactivarToken = ps.executeUpdate();
			if (desactivarToken > 0) {
				conexion.commit();
				response.setStatus(true);
				response.setMessage("La token se ha desactivado con éxito.");
			} else {
				response.setStatus(false);
				response.setMessage("Error al desactivar el token, no se ha encontrado ningún registro.");
				conexion.rollback();
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (conexion != null) {
					conexion.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		jsonObjDesactivarToken = new JSONObject(response);
		return jsonObjDesactivarToken;
	}

	@Override
	public JSONObject obtenerSedeAreaCargo(JSONObject datos) {
		System.out.println("TokenFichaMysqlDAO: obtenerSedeAreaCargo");
		JSONObject jObject;
		ResponseHelper response = new ResponseHelper();
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		String sql = ""
			+ "select "
			+ "token_ficha.codigo_sede_area codigoSedeArea, "
			+ "token_ficha.codigo_area_cargo codigoAreaCargo, "
			+ "sede.deslocal nombreSede, "
			+ "area.nombre nombreArea, "
			+ "cargo.nombre nombreCargo "
			+ "FROM token_ficha "
			+ "inner join ficha ON ficha.codigo_ficha = token_ficha.codigo_ficha "
			+ "inner join personal ON personal.codper = ficha.codigo_persona "
			+ "inner join sede_area on token_ficha.codigo_sede_area = sede_area.codigo_sede_area "
			+ "inner join locales sede ON sede.codigo_sede = sede_area.codigo_sede "
			+ "inner join area ON area.codigo_area = sede_area.codigo_area "
			+ "inner join area_cargo on token_ficha.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ "inner join cargo ON cargo.codigo_cargo = area_cargo.codigo_cargo "
			+ "where personal.codper = ? "
			+ "order by 1 desc "
                        + "limit 1 ";

		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			ps = cnx.prepareStatement(sql);
			ps.setInt(1, datos.getInt("codigoPersona"));
			rs = ps.executeQuery();
			if (rs.next()) {
				JSONObject jsonObject = new JSONObject();
				jsonObject.put("codigoSedeArea", rs.getInt("codigoSedeArea"));
				jsonObject.put("codigoAreaCargo", rs.getInt("codigoAreaCargo"));
				jsonObject.put("sede", rs.getString("nombreSede"));
				jsonObject.put("area", rs.getString("nombreArea"));
				jsonObject.put("cargo", rs.getString("nombreCargo"));

				response.setStatus(true);
				response.setMessage("Se obtuvo la sede, area y cargo del personal");
				response.setData(new JSONObject().put("datosAdministrativos", jsonObject));

			} else {
				response.setStatus(false);
				response.setMessage("No se encontraron sede - area - cargos del personal.");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jObject = new JSONObject(response);
		return jObject;
	}
}
