package trismegistoplanilla.mysqldao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.json.JSONArray;
import org.json.JSONObject;
import trismegistoplanilla.dao.VacanteDAO;
import trismegistoplanilla.utilities.ResponseHelper;
import trismegistoplanilla.utilities.StringUtils;
import trismegistoplanilla.utilities.Variables;

public class VacanteMysqlDAO implements VacanteDAO {

	@Override
	public JSONObject listarVacanteDT(JSONObject datos) {
		System.out.println("VacanteMysqlDAO: listarVacanteDT");
		JSONObject jsonReturn = new JSONObject();
		JSONArray jsonArray = new JSONArray();
		String sql;
		int cantidad;
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		int item = Integer.parseInt(datos.getString("start")) + 1;
		String filtro = filtroListarVacanteDT(datos.getJSONObject("json"));
		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			sql = ""
				+ "select top " + datos.getString("length") + " "
				+ "vacante.codigo_vacante codigoVacante, "
				+ "sede.codigo_sede codigoSede, "
				+ "sede.nombre nombreSede, "
				+ "area.codigo_area codigoArea, "
				+ "area.nombre nombreArea, "
				+ "cargo.codigo_cargo codigoCargo, "
				+ "cargo.nombre nombreCargo, "
				+ "vacante.codigo_sede_area codigoSedeArea, "
				+ "vacante.codigo_area_cargo codigoAreaCargo, "
				+ "vacante.cantidad vacanteTotal, "
				+ "( "
				+ "( "
				+ "select count(1) from ficha_laboral fl "
				+ "where fl.estado_registro = 1 "
				+ "and fl.codigo_sede_area = vacante.codigo_sede_area "
				+ "and fl.codigo_area_cargo = vacante.codigo_area_cargo "
				+ ") + ( "
				+ "select count(1) from token_ficha tf "
				+ "where tf.estado_registro = 1 "
				+ "and tf.codigo_sede_area = vacante.codigo_sede_area "
				+ "and tf.codigo_area_cargo = vacante.codigo_area_cargo "
				+ ") "
				+ ") 'vacanteOcupada', "
				+ "( "
				+ "vacante.cantidad - "
				+ "(( "
				+ "select count(1) from ficha_laboral fl "
				+ "where fl.estado_registro = 1 "
				+ "and fl.codigo_sede_area = vacante.codigo_sede_area "
				+ "and fl.codigo_area_cargo = vacante.codigo_area_cargo "
				+ ") + ( "
				+ "select count(1) from token_ficha tf "
				+ "where tf.estado_registro = 1 "
				+ "and tf.codigo_sede_area = vacante.codigo_sede_area "
				+ "and tf.codigo_area_cargo = vacante.codigo_area_cargo "
				+ ")) "
				+ ") 'vacanteDisponible', "
				+ "vacante.fecha_creacion fecha, "
				+ "vacante.estado_registro estado "
				+ "FROM vacante "
				+ "inner join sede_area ON sede_area.codigo_sede_area = vacante.codigo_sede_area "
				+ "inner join sede ON sede.codigo_sede = sede_area.codigo_sede "
				+ "inner join area ON area.codigo_area = sede_area.codigo_area "
				+ "inner join area_cargo ON area_cargo.codigo_area_cargo = vacante.codigo_area_cargo "
				+ "inner join cargo ON cargo.codigo_cargo = area_cargo.codigo_cargo "
				+ "where vacante.codigo_vacante not in "
				+ "( "
				+ "select top " + datos.getString("start") + " x.codigo_vacante from vacante x "
				+ "inner join sede_area ON sede_area.codigo_sede_area = x.codigo_sede_area "
				+ "inner join sede ON sede.codigo_sede = sede_area.codigo_sede "
				+ "inner join area ON area.codigo_area = sede_area.codigo_area "
				+ "inner join area_cargo ON area_cargo.codigo_area_cargo = x.codigo_area_cargo "
				+ "inner join cargo ON cargo.codigo_cargo = area_cargo.codigo_cargo "
				+ "where x.estado_registro = 1 "
				+ filtro
				+ "order by sede.nombre, area.nombre, cargo.nombre asc "
				+ ") "
				+ filtro
				+ "and vacante.estado_registro = 1 "
				+ "order by sede.nombre, area.nombre, cargo.nombre asc";

			ps = cnx.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				JSONObject vacante = new JSONObject();
				vacante.put("item", item++);
				vacante.put("codigoVacante", rs.getInt("codigoVacante"));
				vacante.put("codigoSede", rs.getInt("codigoSede"));
				vacante.put("nombreSede", rs.getString("nombreSede"));
				vacante.put("codigoArea", rs.getInt("codigoArea"));
				vacante.put("nombreArea", rs.getString("nombreArea"));
				vacante.put("codigoCargo", rs.getInt("codigoCargo"));
				vacante.put("nombreCargo", rs.getString("nombreCargo"));
				vacante.put("codigoSedeArea", rs.getInt("codigoSedeArea"));
				vacante.put("codigoAreaCargo", rs.getInt("codigoAreaCargo"));
				vacante.put("vacanteTotal", rs.getInt("vacanteTotal"));
				vacante.put("vacanteOcupada", rs.getInt("vacanteOcupada"));
				vacante.put("vacanteDisponible", rs.getInt("vacanteDisponible"));
				vacante.put("fecha", rs.getDate("fecha"));
				vacante.put("estado", rs.getInt("estado"));
				jsonArray.put(vacante);
			}

			sql
				= "select count(1) cantidad "
				+ "FROM vacante "
				+ "inner join sede_area ON sede_area.codigo_sede_area = vacante.codigo_sede_area "
				+ "inner join sede ON sede.codigo_sede = sede_area.codigo_sede "
				+ "inner join area ON area.codigo_area = sede_area.codigo_area "
				+ "inner join area_cargo ON area_cargo.codigo_area_cargo = vacante.codigo_area_cargo "
				+ "inner join cargo ON cargo.codigo_cargo = area_cargo.codigo_cargo "
				+ "where vacante.estado_registro = 1 "
				+ filtro;

			ps = cnx.prepareStatement(sql);
			rs = ps.executeQuery();
			rs.next();
			cantidad = rs.getInt("cantidad");

			jsonReturn.put("data", jsonArray);
			jsonReturn.put("recordsFiltered", cantidad);
			jsonReturn.put("recordsTotal", cantidad);
			jsonReturn.put("draw", datos.getInt("draw"));
		} catch (SQLException e) {
			e.printStackTrace();
			jsonReturn.put("status", false);
			jsonReturn.put("message", "Error: " + e.getMessage() + "   Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		return jsonReturn;
	}

	private String filtroListarVacanteDT(JSONObject datos) {
		System.out.println("VacanteMysqlDAO: filtroListarVacanteDT");
		String str = "";
		int sede = datos.getInt("codigoSede");
		int area = datos.getInt("codigoArea");
		int cargo = datos.getInt("codigoCargo");

		if (sede != 0) {
			str = " and sede.codigo_sede = " + sede + " ";
		}

		if (area != 0) {
			str = " and sede.codigo_sede = " + sede + " and area.codigo_area = " + area + " ";
		}

		if (cargo != 0) {
			str = " and sede.codigo_sede = " + sede + " and area.codigo_area = " + area + " and cargo.codigo_cargo = " + cargo + " ";
		}

		return str;
	}

	@Override
	public JSONObject registrarVacante(JSONObject datos) {
		System.out.println("VacanteMysqlDAO: registrarVacante");
		JSONObject jOVacante;
		ResponseHelper response = new ResponseHelper();
		String sql;
		int resultado;
		Connection cnx = null;
		PreparedStatement ps = null;
		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			cnx.setAutoCommit(false);
			sql
				= "update vacante "
				+ "set estado_registro = 0 "
				+ "where vacante.estado_registro = 1 "
				+ "and vacante.codigo_sede_area = ? "
				+ "and vacante.codigo_area_cargo = ?";
			ps = cnx.prepareStatement(sql);
			ps.setInt(1, datos.getInt("codigoSedeArea"));
			ps.setInt(2, datos.getInt("codigoAreaCargo"));
			resultado = ps.executeUpdate();
			if (resultado == 1) {
				sql
					= "insert into vacante "
					+ "(codigo_sede_area, codigo_area_cargo, cantidad, fecha_creacion, estado_registro) "
					+ "values (?,?,?,now(),1)";
				ps = cnx.prepareStatement(sql);
				int c = 1;
				ps.setInt(c++, datos.getInt("codigoSedeArea"));
				ps.setInt(c++, datos.getInt("codigoAreaCargo"));
				ps.setInt(c++, datos.getInt("cantidad"));
				resultado = ps.executeUpdate();
				if (resultado == 1) {
					cnx.commit();
					response.setStatus(true);
					response.setMessage("Vacante actualizada!");
				} else {
					response.setStatus(false);
					response.setMessage("No se pudo actualizar la vacante");
				}
			} else {
				response.setStatus(false);
				response.setMessage("No se pudo actualizar la vacante");
			}

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + "   Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jOVacante = new JSONObject(response);
		return jOVacante;
	}

	@Override
	public JSONObject validarVacante(JSONObject datos) {
		System.out.println("VacanteMysqlDAO: validarVacante");
		JSONObject jOVacante;
		ResponseHelper response = new ResponseHelper();
		String sql;
		int vacantesOcupadas, resultado;
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			sql = ""
				+ "select ( "
				+ "(select count(1) "
				+ "from ficha_laboral "
				+ "where ficha_laboral.estado_registro = 1 "
				+ "and ficha_laboral.codigo_sede_area = vacante.codigo_sede_area "
				+ "and ficha_laboral.codigo_area_cargo = vacante.codigo_area_cargo) "
				+ "+ ( "
				+ "select count(1) from token_ficha "
				+ "where token_ficha.estado_registro = 1 "
				+ "and token_ficha.codigo_sede_area = vacante.codigo_sede_area "
				+ "and token_ficha.codigo_area_cargo = vacante.codigo_area_cargo "
				+ ")) vacantesOcupadas "
				+ "FROM vacante "
				+ "where vacante.codigo_sede_area = ? "
				+ "and vacante.codigo_area_cargo = ? "
				+ "and vacante.estado_registro = 1";
			ps = cnx.prepareStatement(sql);
			ps.setInt(1, datos.getInt("codigoSedeArea"));
			ps.setInt(2, datos.getInt("codigoAreaCargo"));
			rs = ps.executeQuery();
			if (rs.next()) {
				vacantesOcupadas = rs.getInt("vacantesOcupadas");
				resultado = datos.getInt("cantidad") - vacantesOcupadas;
				if (resultado >= 0) {
					response.setStatus(true);
					response.setMessage("Puede eliminar vacantes");
				} else {
					response.setStatus(false);
					response.setMessage("No puede eliminar vacantes. Hay cargos ocupandolos.");
				}
			} else {
				response.setStatus(false);
				response.setMessage("No se pudo obtener las vacantes.");
			}

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + "   Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jOVacante = new JSONObject(response);
		return jOVacante;
	}

	@Override
	public JSONObject validarVacanteFicha(JSONObject datos) {
		System.out.println("VacanteMysqlDAO: validarVacanteFicha");
		JSONObject jObject;
		ResponseHelper response = new ResponseHelper();
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		String sql = "";
		int resultado = 0;

		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);

			sql = "SELECT "
				+ "	( "
				+ "		vacante.cantidad - ( "
				+ "			( "
				+ "				SELECT "
				+ "					COUNT(1) "
				+ "				FROM "
				+ "					ficha_laboral fl "
				+ "				WHERE "
				+ "					fl.estado_registro = 1 "
				+ "				AND fl.codigo_sede_area = vacante.codigo_sede_area "
				+ "				AND fl.codigo_area_cargo = vacante.codigo_area_cargo "
				+ "			) + ( "
				+ "				SELECT "
				+ "					COUNT(1) "
				+ "				FROM "
				+ "					token_ficha tf "
				+ "				WHERE "
				+ "					tf.estado_registro = 1 "
				+ "				AND tf.codigo_sede_area = vacante.codigo_sede_area "
				+ "				AND tf.codigo_area_cargo = vacante.codigo_area_cargo "
				+ "			) "
				+ "		) "
				+ "	) disponible "
				+ "FROM "
				+ "	vacante "
				+ "WHERE "
				+ "	vacante.estado_registro = 1 "
				+ "AND codigo_sede_area = ? "
				+ "AND codigo_area_cargo = ?";

			ps = cnx.prepareStatement(sql);
			ps.setInt(1, datos.getInt("codigoSedeArea"));
			ps.setInt(2, datos.getInt("codigoAreaCargo"));
			rs = ps.executeQuery();
			if (rs.next()) {
				resultado = rs.getInt("disponible");
				if (resultado > 0) {
					JSONObject jOResulted = new JSONObject();
					jOResulted.put("getResulted", resultado);
					response.setStatus(true);
					response.setMessage("Si hay vacantes disponibles");
					response.setData(jOResulted);

				} else {
					response.setStatus(false);
					response.setMessage("No tiene vacantes disponibles");
				}
			} else {
				response.setStatus(false);
				response.setMessage("No se encontró la vacante del cargo!");
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + "   Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jObject = new JSONObject(response);
		return jObject;
	}

	@Override
	public JSONObject listarVacanteSimpleTable(JSONObject datos) {
		System.out.println("VacanteMysqlDAO: listarVacanteSimpleTable");
		JSONObject jObject;
		JSONArray data = new JSONArray();
		ResponseHelper response = new ResponseHelper();
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		String sql = ""
			+ "select "
			+ "( "
			+ "select v.codigo_vacante from vacante v "
			+ "where v.estado_registro = 1 "
			+ "and v.codigo_sede_area = sede_area.codigo_sede_area "
			+ "and v.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ ") 'codigoVacante', "
			+ "sede.codigo_sede codigoSede, "
			+ "sede.nombre nombreSede, "
			+ "area.codigo_area codigoArea, "
			+ "area.nombre nombreArea, "
			+ "cargo.codigo_cargo codigoCargo, "
			+ "cargo.nombre nombreCargo, "
			+ "sede_area.codigo_sede_area codigoSedeArea, "
			+ "area_cargo.codigo_area_cargo codigoAreaCargo, "
			+ "( "
			+ "select v.cantidad from vacante v "
			+ "where v.estado_registro = 1 "
			+ "and v.codigo_sede_area = sede_area.codigo_sede_area "
			+ "and v.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ ") 'vacanteTotal', "
			+ "(( "
			+ "select count(1) from ficha_laboral fl "
			+ "where fl.estado_registro = 1 "
			+ "and fl.codigo_sede_area = sede_area.codigo_sede_area "
			+ "and fl.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ ") + ( "
			+ "select count(1) from token_ficha tf "
			+ "where tf.estado_registro = 1 "
			+ "and tf.codigo_sede_area = sede_area.codigo_sede_area "
			+ "and tf.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ ")) 'vacanteOcupada', "
			+ "(( "
			+ "select v.cantidad from vacante v "
			+ "where v.estado_registro = 1 "
			+ "and v.codigo_sede_area = sede_area.codigo_sede_area "
			+ "and v.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ ") - (( "
			+ "select count(1) from ficha_laboral fl "
			+ "where fl.estado_registro = 1 "
			+ "and fl.codigo_sede_area = sede_area.codigo_sede_area "
			+ "and fl.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ ") + ( "
			+ "select count(1) from token_ficha tf "
			+ "where tf.estado_registro = 1 "
			+ "and tf.codigo_sede_area = sede_area.codigo_sede_area "
			+ "and tf.codigo_area_cargo = area_cargo.codigo_area_cargo "
			+ ")) "
			+ ") 'vacanteDisponible' "
			+ "FROM sede_area "
			+ "inner join sede ON sede.codigo_sede = sede_area.codigo_sede "
			+ "inner join area ON area.codigo_area = sede_area.codigo_area "
			+ "inner join area_cargo ON area_cargo.codigo_area = area.codigo_area "
			+ "inner join cargo ON cargo.codigo_cargo = area_cargo.codigo_cargo "
			+ "order by sede.nombre, area.nombre, cargo.nombre";
		int item = 1;
		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			ps = cnx.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				JSONObject jOVacante = new JSONObject();
				jOVacante.put("item", item++);
				jOVacante.put("codigoVacante", StringUtils.isNullOrEmpty(rs.getString("codigoVacante")) ? "" : rs.getInt("codigoVacante"));
				jOVacante.put("codigoSede", rs.getInt("codigoSede"));
				jOVacante.put("nombreSede", rs.getString("nombreSede"));
				jOVacante.put("codigoArea", rs.getInt("codigoArea"));
				jOVacante.put("nombreArea", rs.getString("nombreArea"));
				jOVacante.put("codigoCargo", rs.getInt("codigoCargo"));
				jOVacante.put("nombreCargo", rs.getString("nombreCargo"));
				jOVacante.put("codigoSedeArea", rs.getInt("codigoSedeArea"));
				jOVacante.put("codigoAreaCargo", rs.getInt("codigoAreaCargo"));

				jOVacante.put("vacanteTotal", StringUtils.isNullOrEmpty(rs.getString("vacanteTotal")) ? "" : rs.getInt("vacanteTotal"));
				jOVacante.put("vacanteOcupada", StringUtils.isNullOrEmpty(rs.getString("codigoVacante")) ? "" : rs.getInt("vacanteOcupada"));
				jOVacante.put("vacanteDisponible", StringUtils.isNullOrEmpty(rs.getString("vacanteDisponible")) ? "" : rs.getInt("vacanteDisponible"));

				data.put(jOVacante);
			}

			response.setStatus(true);
			response.setMessage("Se ha listado las vacantes correctamente!");
			response.setData(new JSONObject().put("vacantes", data));

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + "   Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jObject = new JSONObject(response);
		return jObject;
	}

	@Override
	public JSONObject generarVacante(JSONObject datos) {
		System.out.println("VacanteMysqlDAO: generarVacante");
		JSONObject jObject;
		ResponseHelper response = new ResponseHelper();
		Connection cnx = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		String sql = "insert into vacante (codigo_sede_area,codigo_area_cargo,cantidad,fecha_creacion,estado_registro) "
			+ "VALUES (?,?,?,now(),1)";
		int resultado;
		try {
			cnx = MysqlDAOFactory.obtenerConexion(Variables.MYSQL_NUEVO_BD_NAME);
			cnx.setAutoCommit(false);
			ps = cnx.prepareStatement(sql);
			int c = 1;
			ps.setInt(c++, datos.getInt("codigoSedeArea"));
			ps.setInt(c++, datos.getInt("codigoAreaCargo"));
			ps.setInt(c++, datos.getInt("cantidad"));

			resultado = ps.executeUpdate();

			if (resultado == 1) {
				response.setStatus(true);
				response.setMessage("Se generó la vacante correctamente!");
				cnx.commit();
			} else {
				response.setStatus(false);
				response.setMessage("No se pudo generar la vacante");
				cnx.rollback();
			}
		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + "   Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jObject = new JSONObject(response);
		return jObject;
	}
}
