package trismegistoplanilla.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.json.JSONArray;
import org.json.JSONObject;
import pe.siso.webservicesseguridad.webservices.UsuarioBean;
import trismegistoplanilla.beans.EstadoFichaBean;
import trismegistoplanilla.beans.PersonaBean;
import trismegistoplanilla.beans.TokenFichaBean;
import trismegistoplanilla.servicesMysql.PersonaService;
import trismegistoplanilla.servicesMysql.TipoDocumentoService;
import trismegistoplanilla.utilities.ParamsValidation;

public class PersonaServlet extends HttpServlet {

	private static final long serialVersionUID = 767323838778604646L;

	HttpSession session = null;
	String mensaje = "";

	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		session = request.getSession();

		String accion = request.getParameter("accion");
		if (accion == null) {

		} else {
			switch (accion) {
				case "validarExistenciaNumeroDocumento":
					validarExistenciaNumeroDocumento(request, response);
					break;
				case "validarExistenciaCorreoElectronico":
					validarExistenciaCorreoElectronico(request, response);
					break;
				case "registrarNuevaFicha":
					registrarNuevaFicha(request, response);
					break;
				case "verificarPersona":
					verificarPersona(request, response);
					break;
				default:
					break;
			}
		}

	}

	private void validarExistenciaNumeroDocumento(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setContentType("application/json");
		PrintWriter out = response.getWriter();
		JSONObject JOPersona = new JSONObject();
		if (request.getParameter("json") == null) {
			JOPersona.put("status", false);
			JOPersona.put("message", "El parámetro llegó nulo");
		} else {
			String data = request.getParameter("json");
			JSONObject json = new JSONObject(data);

			// validar numero de documento
			if (!ParamsValidation.validaTipoEntrada(json.getString("tipoEntrada"), json.getString("numeroDocumento"))) {
				JOPersona.put("status", false);
				JOPersona.put("message", "El parámetro llegó nulo");
			}

			if (!ParamsValidation.validaLongitudNumeroDocumento(json.getInt("longitud"), json.getString("numeroDocumento"))) {
				JOPersona.put("status", false);
				JOPersona.put("message", "El parámetro llegó nulo");
			}

			PersonaService service = new PersonaService();
			PersonaBean p = new PersonaBean();
			p.setCodigoTipoDocumento(json.getInt("codigoTipoDocumento"));
			p.setNumeroDocumento(json.getString("numeroDocumento"));
			JOPersona = service.validarExistenciaNumeroDocumento(p);
		}
		out.print(JOPersona);
	}

	private void validarExistenciaCorreoElectronico(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setContentType("application/json");
		PrintWriter out = response.getWriter();
		JSONObject jsonObjValidarExistenciaCorreoElectronico = null;
		if (request.getParameter("correoElectronico") != null) {
			String correoElectronico = request.getParameter("correoElectronico");
			boolean validarCorreoElectronico = ParamsValidation.validaCorreo(correoElectronico);
			if (validarCorreoElectronico) {
				PersonaService service = new PersonaService();
				PersonaBean p = new PersonaBean();
				p.setCorreo(correoElectronico);
				jsonObjValidarExistenciaCorreoElectronico = service.validarExistenciaCorreoElectronico(p);
				out.print(jsonObjValidarExistenciaCorreoElectronico);
			} else {
				jsonObjValidarExistenciaCorreoElectronico = new JSONObject();
				jsonObjValidarExistenciaCorreoElectronico.put("status", false);
				jsonObjValidarExistenciaCorreoElectronico.put("message", "Ha sucedido un error, el dato ingresado no es un correo electrónico");
				out.print(jsonObjValidarExistenciaCorreoElectronico);
			}
		} else {
			jsonObjValidarExistenciaCorreoElectronico = new JSONObject();
			jsonObjValidarExistenciaCorreoElectronico.put("status", false);
			jsonObjValidarExistenciaCorreoElectronico.put("message", "Ha sucedido un error, el dato llegó nulo");
			out.print(jsonObjValidarExistenciaCorreoElectronico);
		}

	}

	private void registrarNuevaFicha(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setContentType("application/json");
		PrintWriter out = response.getWriter();
		JSONObject JOFicha = new JSONObject();

		if (request.getParameter("json") == null) {
			JOFicha.put("status", false);
			JOFicha.put("message", "El parámetro llegó nulo");
		} else {
			String data = request.getParameter("json");
			JSONObject json = new JSONObject(data);

			HttpSession sessionUser = request.getSession();
			UsuarioBean usuario = (UsuarioBean) sessionUser.getAttribute("usuario");

			// -------------------------------------------------------- //
			int codigoUsuario = usuario.getCodigoUsuario();
			EstadoFichaBean ef = new EstadoFichaBean();
			ef.setCodigoUsuario(codigoUsuario);

			// -------------------------------------------------------- //
			int codigoSedeArea = json.getInt("codigoSedeArea");
			int codigoAreaCargo = json.getInt("codigoAreaCargo");
			TokenFichaBean tf = new TokenFichaBean();
			tf.setCodigoSedeArea(codigoSedeArea);
			tf.setCodigoAreaCargo(codigoAreaCargo);

			// -------------------------------------------------------- //
			int codigoTipoDocumento = json.getJSONObject("tipoDocumento").getInt("codigoTipoDocumento");
			String numeroDocumento = json.getJSONObject("tipoDocumento").getString("numeroDocumento");
			int tieneCorreo = 0;
			String correo = "";
			if (json.getBoolean("tieneCorreo")) {
				tieneCorreo = 0;
				correo = json.getString("correo");
			} else {
				tieneCorreo = 1;
				correo = json.getString("correoPorDefecto");
			}

			PersonaBean p = new PersonaBean();
			p.setCodigoTipoDocumento(codigoTipoDocumento);
			p.setNumeroDocumento(numeroDocumento);
			p.setCorreo(correo);
			p.setIsDefaultMail(tieneCorreo);
			p.setApellidoPaterno(null);
			p.setApellidoMaterno(null);
			p.setNombre(null);

			// tipo de ducumento  = dni
			if (json.getJSONObject("tipoDocumento").getInt("codigoTipoDocumento") == 1 && json.getBoolean("validadoPoReniec") && !json.getBoolean("errorReniec")) {
				JSONObject persona = json.getJSONObject("persona");
				String apellidoPaterno = persona.getString("apellidoPaterno");
				String apellidoMaterno = persona.getString("apellidoMaterno");
				String nombres = persona.getString("nombres");
				String dni = persona.getString("dni");
				p.setApellidoPaterno(apellidoPaterno);
				p.setApellidoMaterno(apellidoMaterno);
				p.setNombre(nombres);
				p.setNumeroDocumento(dni);
			} else if (json.getJSONObject("tipoDocumento").getInt("codigoTipoDocumento") == 1 && !json.getBoolean("validadoPoReniec") && json.getBoolean("errorReniec")) {
				JSONObject persona = json.getJSONObject("serviceFailure");
				String apellidoPaterno = persona.getString("apellidoPaterno");
				String apellidoMaterno = persona.getString("apellidoMaterno");
				String nombres = persona.getString("nombres");
				String dni = persona.getString("dni");
				p.setApellidoPaterno(apellidoPaterno);
				p.setApellidoMaterno(apellidoMaterno);
				p.setNombre(nombres);
				p.setNumeroDocumento(dni);
			}

			PersonaService service = new PersonaService();
			JOFicha = service.registrarPersona(p, ef, tf);
		}

		out.print(JOFicha);
	}

	private void verificarPersona(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String codigoVerificacion = request.getParameter("codigoVerificacion");
		String numeroDocumento = request.getParameter("numeroDocumento");

		// verificamos que los parametros enviados desde el jsp no sean nulos
		if (codigoVerificacion != null && numeroDocumento != null) {
			// validar que los parametros sean solo alfanumericos
			if (ParamsValidation.validaSoloAlfanumerico(codigoVerificacion) && ParamsValidation.validaSoloAlfanumerico(numeroDocumento)) {
				// obtenemos los valores de session
				TokenFichaBean tf = (TokenFichaBean) session.getAttribute("tokenFicha");
				tf.setCodigoVerificacion(codigoVerificacion);

				// instanciar objeto persona y setear valores
				PersonaBean p = new PersonaBean();
				p.setNumeroDocumento(numeroDocumento);

				// instanciar servicio persona        
				PersonaService service = new PersonaService();

				// invocar al metodo de verificacion de persona
				JSONObject jsonObjVerificarPersona = service.verificarPersona(tf, p);

				// validar repuesta del metodo verificar persona
				if (jsonObjVerificarPersona.getBoolean("status")) {
					// obtener codigo tipo de documento          
					TipoDocumentoService serviceTipoDocumento = new TipoDocumentoService();
					JSONObject jsonObjObtenerCodigoTipoDocumento = serviceTipoDocumento.obtenerCodigoTipoDocumento(tf);

					// validar respuesta del metodo obtener codigo tipo documento
					if (jsonObjObtenerCodigoTipoDocumento.getBoolean("status")) {
						int codigoTipoDocumento = jsonObjObtenerCodigoTipoDocumento.getJSONObject("data").getInt("getResultedKey");
						JSONObject jsonObjObtenerCodigoPersona = service.obtenerCodigoPersona(tf);

						// validar respuesta del metodo obtener codigo persona
						if (jsonObjObtenerCodigoPersona.getBoolean("status")) {
							int codigoPersona = jsonObjObtenerCodigoPersona.getJSONObject("data").getInt("getResultedKey");

							// instacia de la clase persona
							p.setCodigoPersona(codigoPersona);
							p.setCodigoTipoDocumento(codigoTipoDocumento);
							JSONObject jsonObjObtenerPersonaSession = service.obtenerPersonaSession(p);

							// validar respuesta del metodo obtener persona session
							if (jsonObjObtenerPersonaSession.getBoolean("status")) {
								JSONObject jsonObjPersona = jsonObjObtenerPersonaSession.getJSONObject("data");
								JSONArray jsonArrPersona = jsonObjPersona.getJSONArray("personas");

								if (jsonArrPersona.getJSONObject(0).isNull("apellidoPaterno")) {
									p.setApellidoPaterno(null);
								} else {
									p.setApellidoPaterno(jsonArrPersona.getJSONObject(0).getString("apellidoPaterno"));
								}
								if (jsonArrPersona.getJSONObject(0).isNull("apellidoMaterno")) {
									p.setApellidoMaterno(null);
								} else {
									p.setApellidoMaterno(jsonArrPersona.getJSONObject(0).getString("apellidoMaterno"));
								}
								if (jsonArrPersona.getJSONObject(0).isNull("nombre")) {
									p.setNombre(null);
								} else {
									p.setNombre(jsonArrPersona.getJSONObject(0).getString("nombre"));
								}
								p.setCorreo(jsonArrPersona.getJSONObject(0).getString("correo"));
								p.setNombreTipoDocumento(jsonArrPersona.getJSONObject(0).getString("nombreTipoDocumento"));

								if (codigoTipoDocumento == 3) {
									p.setRuc(numeroDocumento);
								}
								session.setAttribute("persona", p);
								response.sendRedirect("vistas/ficha.jsp");
							} else {
								mensaje = jsonObjObtenerPersonaSession.getString("message");
								session.setAttribute("errorTokenFicha", mensaje);
								response.sendRedirect("vistas/response/ErrorToken.jsp");
								session.removeAttribute("tokenFicha");
							}
						} else {
							mensaje = jsonObjObtenerCodigoPersona.getString("message");
							session.setAttribute("errorTokenFicha", mensaje);
							response.sendRedirect("vistas/response/ErrorToken.jsp");
							session.removeAttribute("tokenFicha");
						}
					} else {
						mensaje = jsonObjObtenerCodigoTipoDocumento.getString("message");
						session.setAttribute("errorTokenFicha", mensaje);
						response.sendRedirect("vistas/response/ErrorToken.jsp");
						session.removeAttribute("tokenFicha");
					}
				} else {
					mensaje = jsonObjVerificarPersona.getString("message");
					session.setAttribute("errorTokenFicha", mensaje);
					response.sendRedirect("vistas/response/ErrorToken.jsp");
					session.removeAttribute("tokenFicha");
				}
			} else {
				mensaje = "Los valores enviados no son alfanuméricos";
				session.setAttribute("errorTokenFicha", mensaje);
				response.sendRedirect("vistas/response/ErrorToken.jsp");
				session.removeAttribute("tokenFicha");
			}
		} else {
			mensaje = "Los valores son nulos";
			session.setAttribute("errorTokenFicha", mensaje);
			response.sendRedirect("vistas/response/ErrorToken.jsp");
			session.removeAttribute("tokenFicha");
		}

	}

}
