package trismegistoplanilla.sqlserverdao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.json.JSONArray;
import org.json.JSONObject;
import trismegistoplanilla.beans.FormacionAcademicaBean;
import trismegistoplanilla.beans.PersonaBean;
import trismegistoplanilla.dao.FormacionAcademicaDAO;
import trismegistoplanilla.utilities.ResponseHelper;
import trismegistoplanilla.utilities.Variables;

public class FormacionAcademicaSqlserverDAO implements FormacionAcademicaDAO {

	@Override
	public JSONObject obtenerFormacionAcademicaPorPersona(PersonaBean persona) {
		System.out.println("FormacionAcademicaSqlserverDAO: obtenerFormacionAcademicaPorPersona");
		JSONObject jsonObjObtenerFormacionAcademicaPorPersona = null;
		JSONArray jsonArrayObtenerFormacionAcademicaPorPersona = new JSONArray();
		ResponseHelper response = new ResponseHelper();
		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection cnx = null;

		try {
			cnx = SqlserverDAOFactory.obtenerConexion(Variables.BD_NAME);
			String sql
				= "SELECT "
				+ "formacionacademica.nombre_centro_estudio centroEstudios, "
				+ "nivelestudio.nombre nivelEstudio, "
				+ "estadoestudio.nombre estadoEstudio, "
				+ "Format(formacionacademica.fecha_inicio, 'dd/MM/yyyy') fechaInicio, "
				+ "isnull(Format(formacionacademica.fecha_fin, 'dd/MM/yyyy'),'-') fechaFin, "
				+ "Isnull(formacionacademica.documento_adjunto, 'NO TIENE DOCUMENTO ADJUNTO') documentoAdjunto,"
				+ "IsNull(formacionacademica.carrera_profesional,'-') carreraProfesional, "
				+ "formacionacademica.sector_institucion sectorInstitucion, "
				+ "Isnull(formacionacademica.numero_colegiatura, '-') numeroColegiatura, "
				+ "Isnull(formacionacademica.observacion, '-') observacion "
				+ "FROM formacion_academica formacionacademica "
				+ "INNER JOIN nivel_estado nivelestado ON nivelestado.codigo_nivel_estado = formacionacademica.codigo_nivel_estado "
				+ "INNER JOIN estado_estudio estadoestudio ON estadoestudio.codigo_estado_estudio = nivelestado.codigo_estado_estudio "
				+ "INNER JOIN nivel_estudio nivelestudio ON nivelestudio.codigo_nivel_estudio = nivelestado.codigo_nivel_estudio "
				+ "where formacionacademica.codigo_persona = ?";

			ps = cnx.prepareStatement(sql);
			ps.setInt(1, persona.getCodigoPersona());
			rs = ps.executeQuery();
			while (rs.next()) {
				FormacionAcademicaBean formacionAcademica = new FormacionAcademicaBean();
				formacionAcademica.setNombreCentroEstudios(rs.getString("centroEstudios"));
				formacionAcademica.setNivelEstudio(rs.getString("nivelEstudio"));
				formacionAcademica.setEstadoEstudio(rs.getString("estadoEstudio"));
				formacionAcademica.setFechaInicio(rs.getString("fechaInicio"));
				formacionAcademica.setFechaFin(rs.getString("fechaFin"));
				formacionAcademica.setDocumentoAdjunto(rs.getString("documentoAdjunto"));
				formacionAcademica.setNombreCarreraProfesional(rs.getString("carreraProfesional"));
				formacionAcademica.setSectorInstitucion(rs.getString("sectorInstitucion"));
				formacionAcademica.setNumeroColegiatura(rs.getString("numeroColegiatura"));
				formacionAcademica.setObservacion(rs.getString("observacion"));
				JSONObject jsonObjFormacionAcademica = new JSONObject(formacionAcademica);
				jsonArrayObtenerFormacionAcademicaPorPersona.put(jsonObjFormacionAcademica);
			}

			JSONObject jsonObjFormacionAcademica = new JSONObject();
			jsonObjFormacionAcademica.put("formacionacademica", jsonArrayObtenerFormacionAcademicaPorPersona);

			response.setStatus(true);
			response.setMessage("Se ha listado la formacion academica correctamente");
			response.setData(jsonObjFormacionAcademica);

		} catch (SQLException e) {
			e.printStackTrace();
			response.setStatus(false);
			response.setMessage("Error: " + e.getMessage() + " \n Error Code: [" + e.getErrorCode() + "]");
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (ps != null) {
					ps.close();
				}
				if (cnx != null) {
					cnx.close();
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}

		jsonObjObtenerFormacionAcademicaPorPersona = new JSONObject(response);
		return jsonObjObtenerFormacionAcademicaPorPersona;
	}

}
